use axum::{
    extract::{Path, Query, State},
    response::Json,
    routing::{get, post},
    Router,
};
use serde::Deserialize;
use validator::Validate;

use crate::{
    error::{AppError, AppResult},
    models::*,
    services::GachaService,
    AppState,
};

pub fn create_routes() -> Router<AppState> {
    Router::new()
        .route("/draw", post(draw_card))
        .route("/user/:user_did", get(get_user_cards))
        .route("/unique", get(get_unique_registry))
        .route("/stats", get(get_gacha_stats))
        .route("/master", get(get_card_master))
}

/// Draw a card from gacha system
async fn draw_card(
    State(state): State<AppState>,
    Json(request): Json<CardDrawRequest>,
) -> AppResult<Json<CardDrawResponse>> {
    // Validate request
    request.validate().map_err(|e| AppError::validation(e.to_string()))?;
    
    let gacha_service = GachaService::new(state.settings.gacha_config());
    
    let result = gacha_service
        .draw_card(&state.db, &request.user_did, request.is_paid, request.pool_id)
        .await?;
    
    Ok(Json(result))
}

#[derive(Deserialize)]
struct UserCardsQuery {
    limit: Option<i32>,
    offset: Option<i32>,
}

/// Get user's card collection
async fn get_user_cards(
    State(state): State<AppState>,
    Path(user_did): Path<String>,
    Query(query): Query<UserCardsQuery>,
) -> AppResult<Json<UserCardCollectionResponse>> {
    let limit = query.limit.unwrap_or(50).min(100); // Max 100 cards per request
    let offset = query.offset.unwrap_or(0);

    // Get user ID from DID
    let user = match &state.db {
        crate::database::Database::Postgres(pool) => {
            sqlx::query_as::<_, User>("SELECT * FROM users WHERE did = $1")
                .bind(&user_did)
                .fetch_optional(pool)
                .await
                .map_err(AppError::Database)?
        }
        crate::database::Database::Sqlite(pool) => {
            sqlx::query_as::<_, User>("SELECT * FROM users WHERE did = ?")
                .bind(&user_did)
                .fetch_optional(pool)
                .await
                .map_err(AppError::Database)?
        }
    };

    let user = user.ok_or_else(|| AppError::not_found("User not found"))?;

    // Get user's cards with master data
    let cards_with_master = match &state.db {
        crate::database::Database::Postgres(pool) => {
            sqlx::query_as::<_, UserCardWithMasterQuery>(
                r#"
                SELECT 
                    uc.id, uc.user_id, uc.card_id, uc.cp, uc.status, 
                    uc.obtained_at, uc.is_unique, uc.unique_id,
                    cm.id as master_id, cm.name, cm.base_cp_min, cm.base_cp_max, cm.color, cm.description
                FROM user_cards uc
                JOIN card_master cm ON uc.card_id = cm.id
                WHERE uc.user_id = $1
                ORDER BY uc.obtained_at DESC
                LIMIT $2 OFFSET $3
                "#
            )
            .bind(user.id)
            .bind(limit as i64)
            .bind(offset as i64)
            .fetch_all(pool)
            .await
            .map_err(AppError::Database)?
        }
        crate::database::Database::Sqlite(pool) => {
            sqlx::query_as::<_, UserCardWithMasterQuery>(
                r#"
                SELECT 
                    uc.id, uc.user_id, uc.card_id, uc.cp, uc.status, 
                    uc.obtained_at, uc.is_unique, uc.unique_id,
                    cm.id as master_id, cm.name, cm.base_cp_min, cm.base_cp_max, cm.color, cm.description
                FROM user_cards uc
                JOIN card_master cm ON uc.card_id = cm.id
                WHERE uc.user_id = ?
                ORDER BY uc.obtained_at DESC
                LIMIT ? OFFSET ?
                "#
            )
            .bind(user.id)
            .bind(limit as i32)
            .bind(offset as i32)
            .fetch_all(pool)
            .await
            .map_err(AppError::Database)?
        }
    };

    let mut cards = Vec::new();
    let mut rarity_breakdown = RarityBreakdown {
        normal: 0,
        rare: 0,
        super_rare: 0,
        kira: 0,
        unique: 0,
    };

    for row in cards_with_master {
        let status = match row.status.as_str() {
            "normal" => CardRarity::Normal,
            "rare" => CardRarity::Rare,
            "super_rare" => CardRarity::SuperRare,
            "kira" => CardRarity::Kira,
            "unique" => CardRarity::Unique,
            _ => CardRarity::Normal,
        };

        // Update rarity breakdown
        match status {
            CardRarity::Normal => rarity_breakdown.normal += 1,
            CardRarity::Rare => rarity_breakdown.rare += 1,
            CardRarity::SuperRare => rarity_breakdown.super_rare += 1,
            CardRarity::Kira => rarity_breakdown.kira += 1,
            CardRarity::Unique => rarity_breakdown.unique += 1,
        }

        cards.push(UserCardWithMaster {
            card: UserCardResponse {
                id: row.id,
                card_id: row.card_id,
                cp: row.cp,
                status,
                skill: None, // TODO: Add skill field to query if needed
                obtained_at: row.obtained_at,
                is_unique: row.is_unique,
                unique_id: row.unique_id,
            },
            master: CardMasterResponse {
                id: row.master_id,
                name: row.name,
                base_cp_min: row.base_cp_min,
                base_cp_max: row.base_cp_max,
                color: row.color,
                description: row.description,
            },
        });
    }

    // Get total count and unique count
    let (total_count, unique_count): (i64, i64) = match &state.db {
        crate::database::Database::Postgres(pool) => {
            sqlx::query_as(
                "SELECT COUNT(*) as total, COUNT(*) FILTER (WHERE is_unique = true) as unique_count FROM user_cards WHERE user_id = $1"
            )
            .bind(user.id)
            .fetch_one(pool)
            .await
            .map_err(AppError::Database)?
        }
        crate::database::Database::Sqlite(pool) => {
            sqlx::query_as(
                "SELECT COUNT(*) as total, SUM(CASE WHEN is_unique = 1 THEN 1 ELSE 0 END) as unique_count FROM user_cards WHERE user_id = ?"
            )
            .bind(user.id)
            .fetch_one(pool)
            .await
            .map_err(AppError::Database)?
        }
    };

    Ok(Json(UserCardCollectionResponse {
        user_did,
        cards,
        total_count: total_count as i32,
        unique_count: unique_count as i32,
        rarity_breakdown,
    }))
}

/// Get global unique card registry
async fn get_unique_registry(
    State(state): State<AppState>,
) -> AppResult<Json<UniqueCardRegistryResponse>> {
    // Get all unique cards with master data and owner info
    let unique_cards = match &state.db {
        crate::database::Database::Postgres(pool) => {
            sqlx::query_as::<_, UniqueCardQuery>(
                r#"
                SELECT 
                    cm.id as card_id,
                    cm.name as card_name,
                    ucr.owner_did,
                    u.handle as owner_handle,
                    ucr.obtained_at
                FROM card_master cm
                LEFT JOIN unique_card_registry ucr ON cm.id = ucr.card_id
                LEFT JOIN users u ON ucr.owner_did = u.did
                ORDER BY cm.id
                "#
            )
            .fetch_all(pool)
            .await
            .map_err(AppError::Database)?
        }
        crate::database::Database::Sqlite(pool) => {
            sqlx::query_as::<_, UniqueCardQuery>(
                r#"
                SELECT 
                    cm.id as card_id,
                    cm.name as card_name,
                    ucr.owner_did,
                    u.handle as owner_handle,
                    ucr.obtained_at
                FROM card_master cm
                LEFT JOIN unique_card_registry ucr ON cm.id = ucr.card_id
                LEFT JOIN users u ON ucr.owner_did = u.did
                ORDER BY cm.id
                "#
            )
            .fetch_all(pool)
            .await
            .map_err(AppError::Database)?
        }
    };

    let mut unique_card_infos = Vec::new();
    let mut available_count = 0;

    for row in unique_cards {
        let is_available = row.owner_did.is_none();
        if is_available {
            available_count += 1;
        }

        unique_card_infos.push(UniqueCardInfo {
            card_id: row.card_id,
            card_name: row.card_name,
            owner_did: row.owner_did,
            owner_handle: row.owner_handle,
            obtained_at: row.obtained_at,
            is_available,
        });
    }

    Ok(Json(UniqueCardRegistryResponse {
        unique_cards: unique_card_infos,
        total_unique_cards: 16, // Total number of card types
        available_unique_cards: available_count,
    }))
}

/// Get gacha statistics
async fn get_gacha_stats(State(state): State<AppState>) -> AppResult<Json<GachaStatsResponse>> {
    let gacha_service = GachaService::new(state.settings.gacha_config());
    let stats = gacha_service.get_gacha_stats(&state.db).await?;
    Ok(Json(stats))
}

/// Get card master data
async fn get_card_master(State(state): State<AppState>) -> AppResult<Json<Vec<CardMasterResponse>>> {
    let cards = match &state.db {
        crate::database::Database::Postgres(pool) => {
            sqlx::query_as::<_, CardMaster>("SELECT * FROM card_master ORDER BY id")
                .fetch_all(pool)
                .await
                .map_err(AppError::Database)?
        }
        crate::database::Database::Sqlite(pool) => {
            sqlx::query_as::<_, CardMaster>("SELECT * FROM card_master ORDER BY id")
                .fetch_all(pool)
                .await
                .map_err(AppError::Database)?
        }
    };

    let card_responses: Vec<CardMasterResponse> = cards
        .into_iter()
        .map(|card| CardMasterResponse {
            id: card.id,
            name: card.name,
            base_cp_min: card.base_cp_min,
            base_cp_max: card.base_cp_max,
            color: card.color,
            description: card.description,
        })
        .collect();

    Ok(Json(card_responses))
}