use anyhow::Result;
use axum::{
    extract::State,
    http::StatusCode,
    response::Json,
    routing::{get, post},
    Router,
};
use serde_json::{json, Value};
use std::net::SocketAddr;
use tower_http::cors::CorsLayer;
use tracing::{info, warn};

mod config;
mod database;
mod models;
mod handlers;
mod services;
mod auth;
mod error;

use config::Settings;
use database::Database;
use error::AppError;

#[derive(Clone)]
pub struct AppState {
    pub db: Database,
    pub settings: Settings,
}

#[tokio::main]
async fn main() -> Result<()> {
    // Initialize tracing with debug level
    tracing_subscriber::fmt()
        .with_env_filter("debug")
        .init();

    println!("🎴 ai.card API Server Starting...");
    println!("===================================");

    // Load configuration
    println!("📁 Loading configuration...");
    let settings = Settings::new()
        .map_err(|e| {
            eprintln!("❌ Failed to load configuration: {}", e);
            anyhow::anyhow!("Failed to load configuration: {}", e)
        })?;

    println!("✅ Configuration loaded successfully");
    println!("📍 Config directory: {}", settings.config_dir.display());
    println!("🌐 Port: {}", settings.port);
    println!("🗄️  Database URL: {}", settings.database_url);

    info!("Starting ai.card API server v{}", env!("CARGO_PKG_VERSION"));

    // Initialize database
    println!("🔗 Connecting to database...");
    let database = Database::connect(&settings.database_url).await
        .map_err(|e| {
            eprintln!("❌ Database connection failed: {}", e);
            e
        })?;
    println!("✅ Database connected successfully");
    
    // Run migrations
    println!("🔄 Running database migrations...");
    database.migrate().await
        .map_err(|e| {
            eprintln!("❌ Database migration failed: {}", e);
            e
        })?;
    println!("✅ Database migrations completed");

    let app_state = AppState {
        db: database,
        settings: settings.clone(),
    };

    // Build application routes
    println!("🛣️  Setting up routes...");
    let app = create_app(app_state).await;
    println!("✅ Routes configured");

    // Start server
    let addr = SocketAddr::from(([0, 0, 0, 0], settings.port));
    println!("🚀 Starting server on {}", addr);
    println!("🔗 Health check: http://localhost:{}/health", settings.port);
    println!("📡 API endpoints: http://localhost:{}/api/v1", settings.port);
    println!("🎮 Card endpoints:");
    println!("   - POST /api/v1/cards/draw - Draw card");
    println!("   - GET  /api/v1/cards/user/{{did}} - Get user cards");
    println!("   - GET  /api/v1/cards/unique - Get unique registry");
    println!("   - GET  /api/v1/cards/stats - Get gacha stats");
    println!("   - GET  /api/v1/cards/master - Get card master");
    println!("===================================");
    
    info!("ai.card API server listening on {}", addr);
    
    let listener = tokio::net::TcpListener::bind(addr).await
        .map_err(|e| {
            eprintln!("❌ Failed to bind to address {}: {}", addr, e);
            e
        })?;
    
    println!("🎉 Server started successfully! Press Ctrl+C to stop.");
    
    axum::serve(listener, app).await
        .map_err(|e| {
            eprintln!("❌ Server error: {}", e);
            e
        })?;

    Ok(())
}

async fn create_app(state: AppState) -> Router {
    Router::new()
        // Health check
        .route("/health", get(health_check))
        
        // API v1 routes
        .nest("/api/v1", create_api_routes())
        
        // CORS middleware
        .layer(CorsLayer::permissive())
        
        // Application state
        .with_state(state)
}

fn create_api_routes() -> Router<AppState> {
    Router::new()
        // Authentication routes
        .nest("/auth", handlers::auth::create_routes())
        
        // Card routes
        .nest("/cards", handlers::cards::create_routes())
        
        // Sync routes
        .nest("/sync", handlers::sync::create_routes())
}

async fn health_check() -> Result<Json<Value>, AppError> {
    Ok(Json(json!({
        "status": "healthy",
        "service": "ai.card",
        "version": env!("CARGO_PKG_VERSION"),
        "timestamp": chrono::Utc::now().to_rfc3339()
    })))
}